const _name = "anduin";

const Discord = require("discord.js");

const config = require('wowjp-bots/src/config.json');
const trans = require('wowjp-bots/src/trans.json');
const localConfig = require('./config.json');
const localTrans = require('./trans.json');

const cron = require('node-schedule');

var commands = trans.commands;
var roles = trans.roles;

class Bot {
    constructor() {
        this.bot = new Discord.Client();
    }

    botOn() {
        this.bot.on("ready", () => {
            console.log("Бот " + _name + " успешно запущен!");

            this.globalChannel = this.bot.channels.get(config.channels.general);
            this.hordeChannel = this.bot.channels.get(config.channels.horde);
            this.allianceChannel = this.bot.channels.get(config.channels.alliance);

            this.cronTimer();
    });

        this.userMessageHandler();

        this.bot.login(config.token[_name]);
    }

    userMessageHandler() {
        this.bot.on("message", (message) => {
            if(message.channel.type != 'dm') {
                if(message.author.bot) return;

                if(message.channel.id == this.allianceChannel.id) {
                    switch (message.content) {
                        case commands.general.phrase.title:
                            message.channel.send(this.randPhrase(localTrans.phrases));
                            break;
                    }
                }

                switch (message.content) {
                    case commands.alliance.anduin.title:
                        this.onName(message);
                        break;
                    case commands.forTheAlliance.title:
                        if(!this.canModer(message.member))
                            this.forTheAlliance(message);
                        else
                            message.reply("Пользователи, находящиеся в группе Администраторы или Модераторы, не могут присоединится ни к одной из фракций!");
                        break;
                    case commands.forTheHorde.title:
                        this.forTheHorde(message);
                        break;

                    case commands.general.help.title:
                        this.help(message);
                        break;
                }
        }
    });

    }

    help(message) {
        if(this.isAlliance(message.member)) {
            var text = "**Команды Альянса:** \n";

            for(var key in commands.alliance) {
                text = text + commands.alliance[key].title + ' - ' + commands.alliance[key].descr + '\n';
            }

            this.allianceChannel.send(text);
        }

    }

    forTheAlliance(message) {
        if(message.channel.id == this.allianceChannel.id) return;

        if(this.isAlliance(message.member))
            message.reply(localTrans.roles.fail);
        else if(!this.isAlliance(message.member) && !this.isHorde(message.member)) {
            this.userAddRole(message, roles.alliance);
            message.reply(localTrans.roles.success);
        }
    }

    forTheHorde(message) {
        if(this.isAlliance(message.member))
            message.reply(localTrans.roles.horde);

    }

    isHorde(member) {
        if(member.roles.find("name", roles.horde))
            return true;
        else
            return false;
    }

    isAlliance(member) {
        if(member.roles.find("name", roles.alliance))
            return true;
        else
            return false;
    }

    //Admin & Moder functions

    userAddRole(message, roleName) {
        var role = message.guild.roles.find("name", roleName);
        message.member.addRole(role).catch(console.error);
    }

    canModer(member) {
        if(member.roles.find("name", roles.moder) || member.roles.find("name", roles.admin))
            return true;
        else
            return false;
    }

    isAdmin(member) {
        if(member.roles.find("name", roles.admin))
            return true;
        else
            return false;
    }

    isAdminChannel(channel) {
        if(channel.name == trans.channels.admin)
            return true;
        else
            return false;
    }

    cronTimer() {
        var self = this;
        var phrase = this.randPhrase(localTrans.phrases);

        var run = cron.scheduleJob('*/59 * * * *', function() {
            phrase = self.randPhrase(localTrans.phrases);
            self.allianceChannel.send(phrase);
        });

        /*var rule = new cron.RecurrenceRule();
         rule.minute = 20;*/
    }

    onName(message) {
        var phrase = this.randPhrase(localTrans.onName);
        message.channel.send(phrase);
    }

    randPhrase(type) {
        var length = type.length;
        var rand = Math.floor(Math.random()*length);

        var phrase = type[rand];

        return phrase;
    }
}

module.exports = Bot;