const _name = "keeper";

const Discord = require("discord.js");

const config = require('wowjp-bots/src/config.json');
const trans = require('wowjp-bots/src/trans.json');
const localConfig = require('./config.json');
const localTrans = require('./trans.json');

const fs = require("fs");
const cron = require('node-schedule');

const mysql = require("promise-mysql");
const con = mysql.createPool(config.database);

var commands = trans.commands;
var roles = trans.roles;

class Bot {
    constructor() {
        this.bot = new Discord.Client();
    }

    botOn() {
        this.bot.on("ready", () => {
            console.log("Бот " + _name + " успешно запущен!");
            this.globalChannel = this.bot.channels.get(config.channels.general);
            this.hordeChannel = this.bot.channels.get(config.channels.horde);
            this.allianceChannel = this.bot.channels.get(config.channels.alliance);

            this.cronTimer({hour: 19, minute: 0});
        });

        this.userMessageHandler();
        this.adminMessageHandler();

        this.bot.login(config.token[_name]);
    }

    userMessageHandler() {
        var message_count = 0;
        this.bot.on("message", (message) => {
            if(message.channel.type != 'dm') {
                if(message.author.bot) return;

                this.userDataGet(message);

                switch (message.content) {
                    case commands.general.keeper.title:
                        this.onName(message);
                        break;
                    case commands.general.phrase.title:
                        if(message.channel.id != this.allianceChannel.id || message.channel.id != this.hordeChannel.id)
                            message.channel.send(this.randPhrase(localTrans.phrases));
                        break;
                    case commands.general.help.title:
                        this.help(message);
                        break;
                    case commands.general.info.title:
                        this.userDataInfo(message);
                }
            }
        });

        this.bot.on('guildMemberAdd', member => {
            member.guild.defaultChannel.send(localTrans.greet[0] + member + "! \n" + trans.rules[0]);
        });
    }

    adminMessageHandler() {
        this.bot.on("message", (message) => {
            if(message.channel.type != 'dm' && this.canModer(message.member)) {

                if(message.content.startsWith(commands.moder.kick.title))
                    this.userKick(message);
                else if(message.content.startsWith(commands.moder.user_add.title))
                    this.userDataAdd('adminAdd', message);
                else if(message.content.startsWith(commands.moder.info.title))
                    this.userDataInfoAdmin(message);
                else if(message.content.startsWith(commands.moder.usermod.title))
                    this.userDataModify(message);

                if(this.isAdmin(message.member)) {
                    if(message.content.startsWith(commands.admin.user_del.title))
                        this.userDataDel(message);
                }
            }
        });
    }

    userDataGet(message) {
        con.query("SET SESSION wait_timeout = 604800");

        var sql = "SELECT * FROM ?? WHERE ?? = ?";
        var data = ['users', 'user_id', message.author.id];
        var self = this;

        con.query(sql, data).then(function(data) {
            self.userDataAddXp(data[0], 1);
        }).catch(function(err) {
            self.userDataAdd('add', message);
        });

    }

    //Administrative functions
    userDataAdd(action, message) {
        var self = this;

        if(action === 'add') {
            var user = message.author;
            localConfig.users_template.user_id = user.id;
            localConfig.users_template.name = user.username;

            var textSuccess = `Пользователь ${user.username} (${user}) был успешно добавлен в базу данных.`;
            var textErr = `Пользователь ${user} уже есть в базе данных (БОТ)`;

            var sql = "INSERT INTO ?? SET ?";
            var data = ['users', localConfig.users_template];

            con.query(sql, data).then(function(data) {
                if(data.affectedRows) self.writeToLog(textSuccess + ` ID - ${data.insertId}`);
            }).catch(function(err) {
                self.writeToLog(err + "\n func - userDataAdd in add");
            });
        } else if(action === 'adminAdd') {
            message.mentions.users.map(user => {
                var textSuccess = `Пользователь ${user.username} (${user}) был успешно добавлен в базу данных модератором ${message.author.username}!`;
                var textErr = `Пользователь ${user} уже есть в базе данных`;

                var sql = "SELECT * FROM ?? WHERE ?? = ?";
                var val = ['users', 'user_id', user.id];

                con.query(sql, val).then(function(data) {
                    if(!data[0].id) return false;
                    else if(data[0].id) message.reply(`Пользователь ${user} уже есть в базе данных!`);
                }).catch(function(err) {

                    localConfig.users_template.user_id = user.id;
                    localConfig.users_template.name = user.username;

                    sql = "INSERT INTO ?? SET ?";
                    val = ['users', localConfig.users_template];

                    con.query(sql, val).then(function(data) {
                        if(data.affectedRows) {
                            message.reply(textSuccess);
                            self.writeToLog(textSuccess + ` ID - ${data.insertId}`);
                        }
                    }).catch(function(err) {
                        message.reply(textErr);
                        self.writeToLog(err + "\n func - adminAdd");
                    });
                });
            });
        }
    }

    userDataInfoAdmin(message) {

        var users = new Array();

        message.mentions.users.map(user => {
            users.push(user.id);
        });

        var sql = "SELECT * FROM ?? WHERE ?? IN(?)";
        var val = ['users', 'user_id', users];

        con.query(sql, val).then(function(data) {
            if(!data[0].id) return;
            else {
                for(var i=0; i<data.length; i++) {
                   message.author.send(`
                    Информация о пользователе ${data[i].name}: \n
                    - Уровень: ${data[i].level}; \n
                    - Атака: ${data[i].attack_min} - ${data[i].attack_max} ед.; \n
                    - Магия: ${data[i].spell_attack_min} - ${data[i].spell_attack_max} ед.; \n
                    - Мана: ${data[i].mp}/${data[i].mp_max}; \n
                    - Скорость атаки: физическая - ${data[i].attack_delay} милисекунд, магическая - ${data[i].spell_delay} милисекунд; \n
                    - Стоимость заклинаний: ${data[i].spellcost} ед. маны; \n
                    - Опыт: ${data[i].xp}/${data[i].xp_max} ед.; \n
                    - Золото: ${data[i].gold}; \n
                `);
                }
            }
        }).catch(function(err) {self.writeToLog(err + "\n func - userDataInfoAdmin")});
    }

    userDataDel(message) {
        var users = new Array();
        var self = this;

        message.mentions.users.map(user => {
            users.push(user.id);
        });

        var sql = "SELECT * FROM ?? WHERE ?? IN(?)";
        var val = ['users', 'user_id', users];

        con.query(sql, val).then(function(data) {
            if(!data[0].id) return;
            else {
                var usersLog = '';
                for(var i=0;i<data.length;i++) {
                    usersLog += data[i].name + " (" + data[i].user_id + "), "
                }

                var textSuccess = "Следующие пользователи были удалены: " + usersLog;

                sql = "DELETE FROM ?? WHERE ?? IN(?)";

                con.query(sql, val).then(function(data) {
                   message.reply(textSuccess);
                   self.writeToLog(textSuccess);
                }).catch(function(err) {self.writeToLog(err + "\n func - userDataDel in del")});

            }
        }).catch(function(err) {self.writeToLog(err + "\n func - userDataDel in select")});
    }

    userDataModify(message) {
        var args = message.content.split(" ");
        var param = args['1'];
        var commands = ['gold', 'level'];
        var textParamErr = `Параметр ${param} не найден! Пожалуйста, проверьте правильность ввода данных`;

        if(commands.indexOf(param) > -1) {

            var val = 0;
            if (Number.isInteger(parseInt(args['2'])))
                val = parseInt(args['2']);

            var users = new Array();
            var self = this;

            var textErr = `Неверно указано имя пользователя, либо ни один из пользователей не найден!`;

            message.mentions.users.map(user => {
                users.push(user.id);
            });

            var insert = ['users', 'user_id', users];

            if (!users.length == 0) {
                con.query("SELECT * FROM ?? WHERE ?? IN(?)", insert).then(function (data) {
                    if (data.length == 0) message.reply(textErr);
                    else {
                        if(param == 'level') {
                            self.userDataLevelUp(null, data, val);
                        } else {
                            data.forEach(row => {
                                row[param] += val;
                                if (row[param] < 0) row[param] = 0;

                                var textSuccess = `Параметр ${param} пользователя ${row.name} был изменен модератором ${message.author.username} на ${val} ед. и теперь составляет ${row[param]}`;

                                insert = ['users', param, row[param], 'id', row.id];
                                con.query("UPDATE ?? SET ?? = ? WHERE ?? = ?", insert).then(function (data) {
                                    if (!data.affectedRows) return false;
                                    else message.reply(textSuccess);
                                }).catch(function (err) {
                                    self.writeToLog(err + "\n func userModify update")
                                });
                            });
                        }
                    }
                }).catch(function (err) {
                    self.writeToLog(err + "\n func userModify select")
                });
            } else
                message.reply(textErr);
        } else
            message.reply(textParamErr);
    }

    userDataAddXp(data, val) {
        var self = this;
        data.xp += val*data.xp_mult;

        var insert = ['users', {xp: data.xp}, 'id', data.id];
        con.query("UPDATE ?? SET ? WHERE ?? = ?", insert).then(function(data) {
            if(!data.affectedRows) return false;
        }).catch(function(err) {
            console.log(err);
        });

        if(data.xp > data.xp_max) {
            var val = Math.floor(data.xp / data.xp_max);
            this.userDataLevelUp(data, null, val);
        }
    }

    userDataLevelUp(user, users, val) {
        var self = this;
        if(user) {
            user.level += val;
            if(user.level > config.const.max_level)
                user.level = config.const.max_level;
            else if(user.level < 1)
                user.level = 1;

            con.query("SELECT * FROM ?? WHERE ?? = ?", ['stats', 'id', user.level]).then(function(data) {

                if(!data[0].id) return false;
                else {
                    var stats = data[0];
                    var lvlStats = {
                        level: stats.id,
                        xp_max: stats.xp,
                        xp_mult: stats.xp_mult,
                        hp_max: stats.hp,
                        hp: stats.hp,
                        mp_max: stats.mp,
                        mp: stats.mp,
                        attack_min: stats.attack_min,
                        attack_max: stats.attack_max,
                        spell_attack_min: stats.spell_attack_min,
                        spell_attack_max: stats.spell_attack_max,
                        spellcost: stats.spellcost,
                        attack_delay: stats.attack_delay,
                        spell_delay: stats.spell_delay
                    }

                    var val = ['users', lvlStats, 'id', user.id];
                    con.query("UPDATE ?? SET ? WHERE ?? = ?", val).then(function(data) {
                        if(!data.affectedRows) return false;
                        self.globalChannel.send(`Пользователь ${user.name} получает ${stats.id} уровень!`);
                    }).catch(function(err) {
                        self.writeToLog(err + "\n func userDataLevelUp");
                    });
                }
            }).catch(function(err) { self.writeToLog(err + "\n func userDataLevelUp select"); });
        } else if(users) {
            users.forEach(user => {
                user.level += val;
                if(user.level > config.const.max_level)
                    user.level = config.const.max_level;
                else if(user.level < 1)
                    user.level = 1;

                con.query("SELECT * FROM ?? WHERE ?? = ?", ['stats', 'id', user.level]).then(function(data) {

                    if(!data[0].id) return false;
                    else {
                        var stats = data[0];
                        var lvlStats = {
                            level: stats.id,
                            xp_max: stats.xp,
                            xp_mult: stats.xp_mult,
                            hp_max: stats.hp,
                            hp: stats.hp,
                            mp_max: stats.mp,
                            mp: stats.mp,
                            attack_min: stats.attack_min,
                            attack_max: stats.attack_max,
                            spell_attack_min: stats.spell_attack_min,
                            spell_attack_max: stats.spell_attack_max,
                            spellcost: stats.spellcost,
                            attack_delay: stats.attack_delay,
                            spell_delay: stats.spell_delay
                        }

                        var val = ['users', lvlStats, 'id', user.id];
                        con.query("UPDATE ?? SET ? WHERE ?? = ?", val).then(function(data) {
                            if(!data.affectedRows) return false;
                            self.globalChannel.send(`Пользователь ${user.name} получает ${stats.id} уровень!`);
                        }).catch(function(err) {
                            self.writeToLog(err + "\n func userDataLevelUp array");
                        });
                    }
                }).catch(function(err) { console.log (err) });
            });
        }

    }

    writeToLog(text) {
        var date = new Date().toJSON().slice(0,19).replace(/-/g,'/').replace('T', ' ');
        text = date + ': ' + text + '\n';
        fs.appendFile(config.path.log, text, (err) => {
            if (err) console.error(err);
            else console.log(text);
        });
    }

    userAddRole(message, roleName) {
        var role = message.guild.roles.find("name", roleName);
        message.member.addRole(role).catch(console.error);
    }

    userKick(message) {
        message.mentions.users.map(user => {
            if(this.canModer(message.guild.member(user))) {
                message.reply("Вы не можете исключить Модератора или Администратора!");
                this.writeToLog(`!!!WARNING!!!! Неудачная попытка исключения ${user.username} модератором ${message.author.username}`);
            } else {
                message.guild.member(user).kick().catch(console.error);
                var text = `Пользователь ${user.username} (${user}) был успешно исключен из чата модератором ${message.author.username}!`
                message.reply(text);
                this.writeToLog(text);
            }
        });
    }

    canModer(member) {
        if(member.roles.find("name", roles.moder) || member.roles.find("name", roles.admin))
            return true;
        else
            return false;
    }

    isAdmin(member) {
        if(member.roles.find("name", roles.admin))
            return true;
        else
            return false;
    }

    isAdminChannel(channel) {
        if(channel.name == trans.channels.admin)
            return true;
        else
            return false;
    }

    cronTimer(rule) {
        var self = this;
        var phrase = this.randPhrase(localTrans.phrases);

        var run = cron.scheduleJob(rule, function(){
            phrase = self.randPhrase(localTrans.phrases);
            self.globalChannel.send(phrase);
        });

        /*var rule = new cron.RecurrenceRule();
        rule.minute = 20;*/
    }

    onName(message) {
        var phrase = this.randPhrase(localTrans.onName);
        message.channel.send(phrase);
    }

    randPhrase(type) {
        var length = type.length;
        var rand = Math.floor(Math.random()*length);

        var phrase = type[rand];

        return phrase;
    }

    //General functions

    help(message) {
        var text = "**Общие команды:** \n";
        var textAdmin = '';
        var textModer = '';

        if(!this.isHorde(message.member) && !this.isAlliance(message.member) && !this.canModer(message.member))
            text = text + commands.forTheAlliance.title + ' - ' + commands.forTheAlliance.descr + '\n' + commands.forTheHorde.title + ' - ' + commands.forTheHorde.descr + '\n'

        for(var key in commands.general) {
            text = text + commands.general[key].title + ' - ' + commands.general[key].descr + '\n';
        }
        if(this.canModer(message.member) && this.isAdminChannel(message.channel)) {
            if(this.isAdmin(message.member)) {
                textAdmin = '\n**Команды Администратора:** \n';
                for(var key in commands.admin) {
                    textAdmin = textAdmin + commands.admin[key].title + ' - ' + commands.admin[key].descr + '\n';
                }
            }
            textModer = '\n**Команды Модератора:** \n';
            for(var key in commands.moder) {
                textModer = textModer + commands.moder[key].title + ' - ' + commands.moder[key].descr + '\n';
            }
        }

        message.reply(text + textAdmin + textModer);
    }

    userDataInfo(message) {
        var self = this;
        var user = message.author;

        var sql = "SELECT * FROM ?? WHERE ?? = ?";
        var val = ['users', 'user_id', user.id];

        con.query(sql, val).then(function(data) {
            if(!data[0].id) message.reply(`Пользователь ${user.username} не найден в базе данных!`);
            else {
                message.author.send(`
                    Информация о пользователе ${data[0].name}: \n
                    - Уровень: ${data[0].level}; \n
                    - Атака: ${data[0].attack_min} - ${data[0].attack_max} ед.; \n
                    - Магия: ${data[0].spell_attack_min} - ${data[0].spell_attack_max} ед.; \n
                    - Мана: ${data[0].mp}/${data[0].mp_max}; \n
                    - Скорость атаки: физическая - ${data[0].attack_delay} милисекунд, магическая - ${data[0].spell_delay} милисекунд; \n
                    - Стоимость заклинаний: ${data[0].spellcost} ед. маны; \n
                    - Опыт: ${data[0].xp}/${data[0].xp_max} ед.; \n
                    - Золото: ${data[0].gold}; \n
                `);
            }
        }).catch(function(err) {
            message.reply(`Пользователь ${user.username} не найден в базе данных!`);
            self.writeToLog(err + "\n func - userDataInfo")
        });
    }

    isHorde(member) {
        if(member.roles.find("name", roles.horde))
            return true;
        else
            return false;
    }

    isAlliance(member) {
        if(member.roles.find("name", roles.alliance))
            return true;
        else
            return false;
    }
}

module.exports = Bot;